<?php
/**
 * 2021 Addify
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Addify
 *  @copyright 2021 Addify
 *  @license   http://opensource.org/licenses/afl-3.0.php
 */

if (!defined('_PS_VERSION_')) {
    exit;
}
class AddifyfreegiftsPromotionModel extends ObjectModel
{
	public $id;
    public $title;
    public $rule_active;
    public $condition_type;
    public $product;
    public $category;
    public $supplier;
    public $date_add;
    public $date_update;
	public static $definition = array(
        'table'     => 'addifyfreegiftspromotion',
        'primary'   => 'id',
        'multilang' => false,
        'fields'    => array(
            'title'            => array('type' => self::TYPE_HTML),
            'date_add'         => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'date_update'      => array('type' => self::TYPE_DATE, 'validate' => 'isDate'),
            'rule_active'      => array('type' => self::TYPE_BOOL, 'validate' => 'isBool'),
            'condition_type'      => array('type' => self::TYPE_HTML),
            'product'          => array('type' => self::TYPE_HTML),
            'category'          => array('type' => self::TYPE_HTML),
            'supplier'          => array('type' => self::TYPE_HTML),
    ));

	public static function getposts()
	{
		return Db::getInstance()->executeS('
			SELECT *
			FROM `'._DB_PREFIX_.'addifyfreegiftspromotion`');
	}

    public static function setStatus($status, $id)
    {
        if (!$id || empty($status)) {
            return false;
        }
        return (bool)Db::getInstance()->execute('UPDATE '._DB_PREFIX_.self::$definition['table'].'
            SET `'.pSQL($status).'` = !'.pSQL($status).'
            WHERE id = '.(int)$id);
    }

    public static function searchProduct()
    {
        $query = Tools::getValue('q', false);
        if (!$query or $query == '' or Tools::strlen($query) < 1) {
            die();
        }

        /*
         * In the SQL request the "q" param is used entirely to match result in database.
         * In this way if string:"(ref : #ref_pattern#)" is displayed on the return list,
         * they are no return values just because string:"(ref : #ref_pattern#)"
         * is not write in the name field of the product.
         * So the ref pattern will be cut for the search request.
         */
        if ($pos = strpos($query, ' (ref:')) {
            $query = Tools::substr($query, 0, $pos);
        }

        $excludeIds = Tools::getValue('excludeIds', false);
        if ($excludeIds && $excludeIds != 'NaN') {
            $excludeIds = implode(',', array_map('intval', explode(',', $excludeIds)));
        } else {
            $excludeIds = '';
            $excludePackItself= Tools::getValue('packItself', false);
        }

        // Excluding downloadable products from packs because download from pack is not supported
        $excludeVirtuals = (bool)Tools::getValue('excludeVirtuals', true);
        $exclude_packs = (bool)Tools::getValue('exclude_packs', true);

        $context = Context::getContext();

        $sql = 'SELECT p.`id_product`, pl.`link_rewrite`, p.`reference`, pl.`name`, image_shop.`id_image` id_image, il.`legend`, p.`cache_default_attribute`
                FROM `'._DB_PREFIX_.'product` p
                '.Shop::addSqlAssociation('product', 'p').'
                LEFT JOIN `'._DB_PREFIX_.'product_lang` pl ON (pl.id_product = p.id_product AND pl.id_lang = '.(int)$context->language->id.Shop::addSqlRestrictionOnLang('pl').')
                LEFT JOIN `'._DB_PREFIX_.'image_shop` image_shop
                    ON (image_shop.`id_product` = p.`id_product` AND image_shop.cover=1 AND image_shop.id_shop='.(int)$context->shop->id.')
                LEFT JOIN `'._DB_PREFIX_.'image_lang` il ON (image_shop.`id_image` = il.`id_image` AND il.`id_lang` = '.(int)$context->language->id.')
                WHERE (pl.name LIKE \'%'.pSQL($query).'%\' OR p.reference LIKE \'%'.pSQL($query).'%\')'.
            (!empty($excludeIds) ? ' AND p.id_product NOT IN ('.$excludeIds.') ' : ' ').
            (!empty($excludePackItself) ? ' AND p.id_product <> '.$excludePackItself.' ' : ' ').
            ($excludeVirtuals ? 'AND NOT EXISTS (SELECT 1 FROM `'._DB_PREFIX_.'product_download` pd WHERE (pd.id_product = p.id_product))' : '').
            ($exclude_packs ? 'AND (p.cache_is_pack IS NULL OR p.cache_is_pack = 0)' : '').
            ' GROUP BY p.id_product';

        $items = Db::getInstance()->executeS($sql);

        if ($items) {
            foreach ($items as $item) {
                $item['name'] = str_replace('|', '&#124;', $item['name']);
                echo trim($item['name']).(!empty($item['reference']) ? ' (ref: '.$item['reference'].')' : '').'|'.(int)($item['id_product'])."\n";
            }
        }
        die();
    }

    public static function getProductById($id_product)
    {
        return Db::getInstance()->getRow('SELECT `id_product`, `name`
            FROM `'._DB_PREFIX_.'product_lang` WHERE `id_product` = '.(int)$id_product);
    }

     public static function getPromotionRule($id_product, $id_supplier, $id_category)
    {
        $results = Db::getInstance()->executeS('
            SELECT *
            FROM `'._DB_PREFIX_.'addifyfreegiftspromotion` WHERE rule_active = 1');

        if ($results) {
            foreach ($results as $result) {

                $expld_sup_arrys = explode(",",$result['supplier']);
                $expld_pd_arrys = explode(",",$result['product']);
                $expld_categ_arrys = explode(",",$result['category']);

                if ($result['condition_type'] == 0) {
                   return $result;
                }
                elseif ($result['condition_type'] == 1) {
                    if (in_array($id_product, $expld_pd_arrys)) {
                        return $result;
                    }
                }
                elseif ($result['condition_type'] == 2) {
                    
                    if (in_array($id_category, $expld_categ_arrys)) {
                       return $result;
                    }
                    
                }
                elseif ($result['condition_type'] == 3) {

                    if (in_array($id_supplier, $expld_sup_arrys)) {
                        return $result;
                    }
                } 
            }
        }
    }
}